module epnoisemask_mod
    use dal
    use cal
    use param
    use errorHandling
    use save_image
    use dal_aux
    use datafilter
    use array_utils
    use sort_mod
    use eventAttributes ! in saslib/sasdal. For checking which flags have

    implicit none

    private
    integer, parameter :: nxdim = 64 , nydim = 200

    public epnoisemask

contains

    subroutine epnoisemask(inSet,map,mask,savemasks,ccdNo)

        implicit none
        type(DataSetT), intent(IN)                     :: inSet
        integer(kind=Int32), dimension(:,:), intent(IN)  :: map
        integer(kind=Int32), dimension(:,:), intent(OUT) :: mask
        logical(kind=bool), intent(IN)                   :: savemasks
        integer(kind=Int32), intent(IN)                  :: ccdNo

        type(DataSetT) :: tempSet, set
        type(TableT) :: tempTab
        type(ColumnT) :: tempCol, flagCol
        type(ArrayT) ::  arr1
        integer(int32) :: inDataType,  imageShape(0:1), numSelectedPixels
        integer(int8),  pointer :: flagData(:)
        integer(int32),  pointer :: inDimVector(:)
        integer(int32) :: dimvec(2)
        integer(int32), pointer :: inDataInt32(:,:),  tempDataInt32(:)
        integer(int32),  dimension(:,:), pointer :: a1
        real(single),allocatable       :: vector(:)
        real(single)     :: sigmacut
        integer :: mean, medianval

        integer(kind=Int32), dimension(:,:),pointer :: noisymask

        character(256) :: expression
        integer :: i,j, k


        integer(kind=int32), dimension(2) :: npix
        character(len=1024) :: string
        type(DataSetT) :: outfile
        type(arrayT)  :: ima
        integer(kind=Int16), dimension(:,:), pointer :: i2map
        integer(kind=Int32) :: ix, iy

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !! Create noisy pixel mask
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!       if (allocated(noisymask)) deallocate(noisymask)
        allocate(noisymask(nxdim,nydim))
        do j = 1, nydim
          do i = 1, nxdim
            noisymask(i,j) = 0
          enddo
        enddo
        call createNoisyMask(inSet,noisymask)



        dimvec = size(map)

        call message(AppMsg,Sparse,'Creating bright sources mask ...')
        ! create a set
        set = dataSet('temp.ds', CREATE, USE_ENVIRONMENT)
        arr1 = addArray(set, 'test', INTEGER32, dimensions=(/nxdim,nydim/) )
        ! fill with unique numbers
        a1 => int32Array2Data(arr1)
        !mask => int32Array2Data(arr1)
!       if (allocated(mask)) deallocate(mask)
!       allocate(mask(nxdim,nydim))
        do j = 1,nydim
          do i = 1,nxdim
            a1(i-1,j-1) = map(i,j)
          enddo
        enddo

        inDimVector => dimensions(arr1)

        imageShape = inDimVector

        tempSet = dataSet('tempDataSet.ds', TEMP)
        tempTab = addTable(tempSet, 'TEMP_TAB', inDimVector(0)*inDimVector(1))


        inDataType = dataType(arr1)

        tempCol = addColumn(tempTab, 'VAL', inDataType)
        flagCol = addColumn(tempTab, 'FLAG', INTEGER8)
        flagData => int8Data(flagCol)
        flagData = 0
        call release(flagCol)


        if (nullDefined(arr1)) then
            call setNullValue(tempCol, intNullValue(arr1))
        end if
        inDataInt32 => int32Array2Data(arr1)
        tempDataInt32 => int32Data(tempCol)
        tempDataInt32 = reshape(inDataInt32, shape(tempDataInt32))
        k = 0
        do i = 0, size(tempDataInt32)-1
            if (tempDataInt32(i) > 0 ) then
                k = k +1
            endif
        enddo

        if (allocated(vector)) deallocate(vector)
        allocate(vector(k))
        k = 1
        mean = 0
        !! vector not needed. Only in case sigma has to be calculated
        do i = 0, size(tempDataInt32)-1
            if (tempDataInt32(i) > 0 ) then
                vector(k) = tempDataInt32(i)
                mean = mean + tempDataInt32(i)
                k = k + 1
            endif
        enddo

        write(errstr,'(a,t50,2i10.0)') '(before division) mean,k=',mean,k
        call message(AppMsg,Noisy)
        mean = mean / k
        call message(AppMsg,Sparse,'Calculating filter threshold ...')
        write(errstr,'(a,t50,i10.0)')'Pixels mean value : ', mean
        call message(AppMsg,Sparse)
        medianval = median(vector)
        write(errstr,'(a,t50,i10.0)')'Pixels median value : ', medianval
        call message(AppMsg,Sparse)
        sigmacut = realparameter("sigmacut")
        mean = (medianval*sigmacut)+mean
        write(errstr,'(a,t50,i10.0)')'Threshold value : ', mean
        call message(AppMsg,Sparse)

        write(expression,'(a5,i10.0)') "VAL >",mean
        write(errstr,'(a,t45,a)')'Bright source filter expression : ',trim(expression)
        call message(AppMsg,Sparse)

        call initFilter(trim(expression))
        numSelectedPixels = doSelectModify(tempTab, 'FLAG', 0_int16)
        write(errstr,'(a,t50,i10.0)')'Events filtered : ', numSelectedPixels
        call message(AppMsg,Sparse)

        flagCol = column(tempTab, 'FLAG')
        flagData => int8Data(flagCol)


        k = 0
        do j=1,nydim
          do i=1,nxdim
                mask(i,j) = flagData(k)
                k = k +1
                if (noisymask(i,j) > 0) then
                    mask(i,j) = 1
                endif
          enddo
        enddo

        call release(arr1)
        call release(tempSet)
        call release(set)

        if (savemasks) then
          !-----------------------------------
          ! Save mask to file
          !-----------------------------------
          npix(1) = nxdim
          npix(2) = nydim

          write(string,'(a4,i2.2,a4)') 'mask',ccdNo,'.img'
          outfile=dataset(string,Create)
          ima=addArray(outfile,"PRIMARY",Integer16,npix)
          i2map => Int16Array2Data(ima)

          do ix=0,nxdim-1
            do iy=0,nydim-1
                if (mask(ix+1,iy+1) == 1) then
                    i2map(ix,iy)=1
                else
                    i2map(ix,iy)=0
                end if
            end do
          end do
          call release(ima)
          call release(outfile)
        endif


    end subroutine epnoisemask

!=====================================================================
    subroutine createNoisyMask(inSet, mask)
        implicit none
        type(DataSetT), intent(IN) :: inSet
!       character(len=*), intent(IN) :: file_name
        integer(kind=Int32) :: i,iy,nbadpix

        type(DataSetT) :: eventSet
        type(TableT)   :: badpix
        type(ColumnT)  :: bpRawX,bpRawY,yExtent
!-mjf   integer(kind=Int32), intent(inout) , dimension(:,:),pointer:: mask
        integer(kind=Int32), dimension(:,:),pointer :: mask
        integer(kind=Int16), dimension(:), pointer :: bpRawXCol,bpRawYCol,yExtCol


        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !! Create noisy pixel mask
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        !! Create expression mask
        call message(AppMsg,Sparse,'Creating badpix mask ...')

!        eventSet = dataSet(trim(file_name), READ, LOW_MEMORY)

        ! get badpixels and remove from mask
        nbadpix=0
!        if (hasBlock(eventSet,"BADPIX")) then
            badpix = table(inSet,"BADPIX",READ)
!            nbadpix = numberOfRows(badpix)

!            if (nbadpix > 0) then
!                bpRawX = column(badpix,"RAWX",READ)
!                bpRawY = column(badpix,"RAWY",READ)
!                yExtent = column(badpix,"YEXTENT",READ)

!                bpRawXCol => int16Data(bpRawX)
!                bpRawYCol => int16Data(bpRawY)
!                yExtCol => int16Data(yExtent)

!                do i=1,nbadpix
!                    do iy=bpRawYCol(i-1),bpRawYCol(i-1)+yExtCol(i-1)-1
!                        mask(bpRawXCol(i-1),iy)=1
!                    end do
!                end do


!                call release(bpRawX)
!                call release(bpRawY)
!                call release(yExtent)

!            end if

             call release(badpix)
!         end if

!       call release(eventSet)

    end subroutine createNoisyMask


!=====================================================================


end module epnoisemask_mod
