module removenoisyframes_mod
    use dal
    use param
    use errorHandling
    use datafilter
    use updateexpo_mod
    use emutils_mod

    implicit none

contains

    subroutine removenoisyframes(file_name, nevt_cut,ccdNo)
        implicit none
        character(len=*), intent(IN) :: file_name
        type(DataSetT) :: tempSet,noisySet
        type(TableT)   :: tmptevents,noisyevents
        integer(kind=Int32) :: numSelectedPixels
        character(256) :: expression

        integer(kind=Int32), intent(inout) :: ccdNo
        real(kind=double), intent(inout) :: nevt_cut

        write(errstr,'(a,t50)')'Removing noisy frames ...'
        call message(AppMsg,Sparse)
        write(errstr,'(a,t50,a)')'Reading cut keyword temp file : ', file_name
        call message(AppMsg,Sparse)

        noisySet = dataSet(trim(file_name), READ)
        noisyevents = table(noisySet,"EVENTS")
        nevt_cut = real64Attribute(noisyevents, 'NEVT_CUT')
        ccdNo = getCcd(block(noisyevents))
        call release(noisySet)

        write(errstr,'(a,i2.0,a,f10.2)')'CCDNO : ',ccdNo, '     NEVT_CUT :', nevt_cut
        call message(AppMsg,Sparse)

        tempSet = clone(trim(stringParameter('eventset')), 'TMP.ds', TEMP)
        tmptevents = table(tempSet,"EVENTS")

        write(expression,'(a,f4.1)') '(NEVT_FRM >=' ,nevt_cut
        expression = trim(expression)//') && (CCDNR == '
        write(expression,'(a,i2.0)') trim(expression) ,ccdNo
        expression = trim(expression)//' )'

        write(errstr,'(a,t50,a)')'Noisy events expression : ', expression
        call message(AppMsg,Sparse)

        call initFilter(expression)

        numSelectedPixels = doFilter(tmptevents)
        write(errstr,'(a,t50,i10.0)')'Nosiy Events selected : ', numSelectedPixels
        call message(AppMsg,Sparse)

        if(numSelectedPixels > 0 ) then
            write(errstr,'(a)')'Updating exposure keywords ... '
            call message(AppMsg,Sparse)
            call updateExpo(tmptevents,ccdNo)
        else
            call warning("NoisyEventsEmpty","WARNING: No events filtered")
        endif
        call release(tempSet)


    end subroutine removenoisyframes


    subroutine filterEventFile(finalExpression)
        implicit none
        type(DataSetT) :: set
        type(TableT)   :: tevents
        character(256) :: finalExpression
        integer(kind=Int32) :: numSelectedPixels

        set = dataSet(trim(stringParameter('eventset')), MODIFY)
        tevents = table(set,"EVENTS")

        call initFilter(finalExpression)

        write(errstr,'(a,t50,a)')'Clean events expression : ', finalExpression
         call message(AppMsg,Sparse)
        numSelectedPixels = doFilter(tevents)

        write(errstr,'(a,t50,i10.0)')'Clean events selected :', numSelectedPixels
        call message(AppMsg,Sparse)

        call release(set)

    end subroutine filterEventFile

end module removenoisyframes_mod
